@@ -20,12 +20,24 @@
 #include <QHBoxLayout>
 #include <QLabel>
 #include <QLineEdit>
+#include <QMenu>
 #include <QPushButton>
 #include <QVBoxLayout>
 
 using namespace PimCommon;
 BlackListBalooEmailCompletionWidget::BlackListBalooEmailCompletionWidget(QWidget *parent)
     : QWidget(parent)
+    , mNumberOfEmailsFound(new QLabel(this))
+    , mSearchLineEdit(new QLineEdit(this))
+    , mExcludeDomainLineEdit(new QLineEdit(this))
+    , mEmailList(new BlackListBalooEmailList(this))
+    , mSearchButton(new QPushButton(QIcon::fromTheme(QStringLiteral("edit-find")), i18n("Search"), this))
+    , mSelectButton(new QPushButton(i18n("&Select"), this))
+    , mUnselectButton(new QPushButton(i18n("&Unselect"), this))
+    , mShowAllBlackListedEmails(new QPushButton(i18n("Show Blacklisted Emails"), this))
+    , mMoreResult(new QLabel(i18n("<qt><a href=\"more_result\">More result...</a></qt>"), this))
+    , mBlackListWarning(new BlackListBalooEmailWarning(this))
+    , mExcludeEmailFromRegularExpressionLineEdit(new QLineEdit(this))
 {
     auto mainLayout = new QVBoxLayout(this);
 
@@ -36,7 +48,6 @@ BlackListBalooEmailCompletionWidget::BlackListBalooEmailCompletionWidget(QWidget
     lab->setObjectName(QStringLiteral("search_label"));
     searchLayout->addWidget(lab);
 
-    mSearchLineEdit = new QLineEdit(this);
     mSearchLineEdit->setPlaceholderText(i18n("Research is done from 3 characters"));
     mSearchLineEdit->setFocus();
     mSearchLineEdit->setClearButtonEnabled(true);
@@ -45,22 +56,18 @@ BlackListBalooEmailCompletionWidget::BlackListBalooEmailCompletionWidget(QWidget
     connect(mSearchLineEdit, &QLineEdit::returnPressed, this, &BlackListBalooEmailCompletionWidget::slotCheckIfUpdateBlackListIsNeeded);
     searchLayout->addWidget(mSearchLineEdit);
 
-    mSearchButton = new QPushButton(QIcon::fromTheme(QStringLiteral("edit-find")), i18n("Search"), this);
     mSearchButton->setObjectName(QStringLiteral("search_button"));
     connect(mSearchButton, &QAbstractButton::clicked, this, &BlackListBalooEmailCompletionWidget::slotCheckIfUpdateBlackListIsNeeded);
     mSearchButton->setEnabled(false);
     searchLayout->addWidget(mSearchButton);
 
-    mShowAllBlackListedEmails = new QPushButton(i18n("Show Blacklisted Emails"), this);
     mShowAllBlackListedEmails->setObjectName(QStringLiteral("show_blacklisted_email_button"));
     connect(mShowAllBlackListedEmails, &QAbstractButton::clicked, this, &BlackListBalooEmailCompletionWidget::slotShowAllBlacklistedEmail);
     searchLayout->addWidget(mShowAllBlackListedEmails);
 
-    mEmailList = new BlackListBalooEmailList(this);
     mEmailList->setObjectName(QStringLiteral("email_list"));
     mainLayout->addWidget(mEmailList);
 
-    mBlackListWarning = new BlackListBalooEmailWarning(this);
     mBlackListWarning->setObjectName(QStringLiteral("backlistwarning"));
     connect(mBlackListWarning, &BlackListBalooEmailWarning::newSearch, this, &BlackListBalooEmailCompletionWidget::slotSearch);
     connect(mBlackListWarning, &BlackListBalooEmailWarning::saveChanges, this, &BlackListBalooEmailCompletionWidget::slotSaveChanges);
@@ -72,17 +79,14 @@ BlackListBalooEmailCompletionWidget::BlackListBalooEmailCompletionWidget(QWidget
     auto selectElementLayout = new QHBoxLayout;
     searchLineLayout->addLayout(selectElementLayout);
 
-    mSelectButton = new QPushButton(i18n("&Select"), this);
     mSelectButton->setObjectName(QStringLiteral("select_email"));
     connect(mSelectButton, &QAbstractButton::clicked, this, &BlackListBalooEmailCompletionWidget::slotSelectEmails);
     selectElementLayout->addWidget(mSelectButton);
 
-    mUnselectButton = new QPushButton(i18n("&Unselect"), this);
     mUnselectButton->setObjectName(QStringLiteral("unselect_email"));
     connect(mUnselectButton, &QAbstractButton::clicked, this, &BlackListBalooEmailCompletionWidget::slotUnselectEmails);
     selectElementLayout->addWidget(mUnselectButton);
 
-    mMoreResult = new QLabel(i18n("<qt><a href=\"more_result\">More result...</a></qt>"), this);
     mMoreResult->setObjectName(QStringLiteral("moreresultlabel"));
     selectElementLayout->addWidget(mMoreResult);
 
@@ -100,10 +104,9 @@ BlackListBalooEmailCompletionWidget::BlackListBalooEmailCompletionWidget(QWidget
     new KPIM::LineEditCatchReturnKey(mSearchInResultLineEdit, this);
 
     searchLineLayout->addStretch(0);
-    mNumberOfEmailsFound = new QLabel(this);
     mNumberOfEmailsFound->setObjectName(QStringLiteral("numberofemailsfound"));
 
-    searchLineLayout->addWidget(mNumberOfEmailsFound);
+    searchLineLayout->addWidget(mNumberOfEmailsFound, 1);
     searchLineLayout->addWidget(mSearchInResultLineEdit);
 
     auto excludeDomainLayout = new QHBoxLayout;
@@ -114,37 +117,121 @@ BlackListBalooEmailCompletionWidget::BlackListBalooEmailCompletionWidget(QWidget
     excludeDomainLabel->setObjectName(QStringLiteral("domain_label"));
     excludeDomainLayout->addWidget(excludeDomainLabel);
 
-    mExcludeDomainLineEdit = new QLineEdit(this);
     excludeDomainLayout->addWidget(mExcludeDomainLineEdit);
     mExcludeDomainLineEdit->setObjectName(QStringLiteral("domain_lineedit"));
     mExcludeDomainLineEdit->setClearButtonEnabled(true);
     new KPIM::LineEditCatchReturnKey(mExcludeDomainLineEdit, this);
     mExcludeDomainLineEdit->setPlaceholderText(i18n("Separate domain with \'%1\'", QLatin1Char(',')));
 
+    auto excludeEmailRegularExpressionLayout = new QHBoxLayout;
+    excludeEmailRegularExpressionLayout->setContentsMargins({});
+    mainLayout->addLayout(excludeEmailRegularExpressionLayout);
+
+    auto excludeEmailRegularExpressionLabel = new QLabel(i18n("Exclude email with Regular Expression:"), this);
+    excludeEmailRegularExpressionLabel->setObjectName(QStringLiteral("email_regularexpression_label"));
+    excludeEmailRegularExpressionLayout->addWidget(excludeEmailRegularExpressionLabel);
+
+    excludeEmailRegularExpressionLayout->addWidget(mExcludeEmailFromRegularExpressionLineEdit);
+    mExcludeEmailFromRegularExpressionLineEdit->setObjectName(QStringLiteral("exclude_email_lineedit"));
+    mExcludeEmailFromRegularExpressionLineEdit->setClearButtonEnabled(true);
+    new KPIM::LineEditCatchReturnKey(mExcludeEmailFromRegularExpressionLineEdit, this);
+    mExcludeEmailFromRegularExpressionLineEdit->setPlaceholderText(i18n("Separate regular expression with \'%1\'", QLatin1Char(',')));
+
     connect(mEmailList, &QListWidget::itemSelectionChanged, this, &BlackListBalooEmailCompletionWidget::slotSelectionChanged);
     slotSelectionChanged();
+    connect(mEmailList, &QListWidget::customContextMenuRequested, this, &BlackListBalooEmailCompletionWidget::slotCustomContextMenuRequested);
 }
 
 BlackListBalooEmailCompletionWidget::~BlackListBalooEmailCompletionWidget() = default;
 
 void BlackListBalooEmailCompletionWidget::slotSelectionChanged()
 {
-    bool selectionIsNotEmpty = !mEmailList->selectedItems().isEmpty();
+    const bool selectionIsNotEmpty = !mEmailList->selectedItems().isEmpty();
     mSelectButton->setEnabled(selectionIsNotEmpty);
     mUnselectButton->setEnabled(selectionIsNotEmpty);
 }
 
+void BlackListBalooEmailCompletionWidget::slotCustomContextMenuRequested(const QPoint &pos)
+{
+    const bool selectionIsNotEmpty = !mEmailList->selectedItems().isEmpty();
+    if (selectionIsNotEmpty) {
+        QMenu menu(this);
+        menu.addAction(i18n("Select"), this, &BlackListBalooEmailCompletionWidget::slotSelectEmails);
+        menu.addSeparator();
+        menu.addAction(i18n("Unselect"), this, &BlackListBalooEmailCompletionWidget::slotUnselectEmails);
+        menu.exec(mapToGlobal(pos));
+    }
+}
+
 void BlackListBalooEmailCompletionWidget::load()
 {
     KSharedConfig::Ptr config = KSharedConfig::openConfig(QStringLiteral("kpimbalooblacklist"));
     KConfigGroup group(config, "AddressLineEdit");
-    const QStringList lst = group.readEntry("ExcludeDomain", QStringList());
-    mEmailList->setExcludeDomain(lst);
-    mExcludeDomainLineEdit->setText(lst.join(QLatin1Char(',')));
-    mOriginalExcludeDomain = lst;
+
+    const QStringList lstExcludeEmailsRegularExpressions = group.readEntry("ExcludeEmailsRegexp", QStringList());
+    mEmailList->setExcludeEmailsRegularExpressions(lstExcludeEmailsRegularExpressions);
+    mExcludeEmailFromRegularExpressionLineEdit->setText(lstExcludeEmailsRegularExpressions.join(QLatin1Char(',')));
+    mOriginalExcludeEmailRegexp = lstExcludeEmailsRegularExpressions;
+
+    const QStringList lstExcludeDomains = group.readEntry("ExcludeDomain", QStringList());
+    mEmailList->setExcludeDomains(lstExcludeDomains);
+    mExcludeDomainLineEdit->setText(lstExcludeDomains.join(QLatin1Char(',')));
+    mOriginalExcludeDomain = lstExcludeDomains;
     slotSelectionChanged();
 }
 
+void BlackListBalooEmailCompletionWidget::save()
+{
+    const QString excludeEmailsRegexp = mExcludeEmailFromRegularExpressionLineEdit->text().remove(QLatin1Char(' '));
+    const QStringList newExcludeEmailsRegExp = excludeEmailsRegexp.split(QLatin1Char(','), Qt::SkipEmptyParts);
+    bool needToSave = (mOriginalExcludeEmailRegexp != newExcludeEmailsRegExp);
+
+    const QString excludeDomains = mExcludeDomainLineEdit->text().remove(QLatin1Char(' '));
+    const QStringList newExcludeDomain = excludeDomains.split(QLatin1Char(','), Qt::SkipEmptyParts);
+    needToSave |= (mOriginalExcludeDomain != newExcludeDomain);
+
+    KSharedConfig::Ptr config = KSharedConfig::openConfig(QStringLiteral("kpimbalooblacklist"));
+    KConfigGroup group(config, "AddressLineEdit");
+    const QHash<QString, bool> result = mEmailList->blackListItemChanged();
+    if (!result.isEmpty()) {
+        needToSave = true;
+        QStringList blackList = group.readEntry("BalooBackList", QStringList());
+        PimCommon::BlackListBalooEmailUtil util;
+        util.initialBlackList(blackList);
+        util.setNewBlackList(result);
+        blackList = util.createNewBlackList();
+        group.writeEntry("BalooBackList", blackList);
+    }
+    if (needToSave) {
+        group.writeEntry("ExcludeEmailsRegexp", newExcludeEmailsRegExp);
+        mEmailList->setExcludeEmailsRegularExpressions(newExcludeEmailsRegExp);
+        mOriginalExcludeEmailRegexp = newExcludeEmailsRegExp;
+
+        group.writeEntry("ExcludeDomain", newExcludeDomain);
+        mEmailList->setExcludeDomains(newExcludeDomain);
+        mOriginalExcludeDomain = newExcludeDomain;
+        group.sync();
+    }
+    config->reparseConfiguration();
+}
+
+void BlackListBalooEmailCompletionWidget::slotSaveChanges()
+{
+    // TODO avoid to save a lot.
+    const QHash<QString, bool> result = mEmailList->blackListItemChanged();
+    KSharedConfig::Ptr config = KSharedConfig::openConfig(QStringLiteral("kpimbalooblacklist"));
+    KConfigGroup group(config, "AddressLineEdit");
+    QStringList blackList = group.readEntry("BalooBackList", QStringList());
+    PimCommon::BlackListBalooEmailUtil util;
+    util.initialBlackList(blackList);
+    util.setNewBlackList(result);
+    blackList = util.createNewBlackList();
+    group.writeEntry("BalooBackList", blackList);
+    group.sync();
+    mEmailList->setEmailBlackList(blackList);
+    slotSearch();
+}
+
 void BlackListBalooEmailCompletionWidget::slotUnselectEmails()
 {
     const QList<QListWidgetItem *> lstSelected = mEmailList->selectedItems();
@@ -205,23 +292,6 @@ void BlackListBalooEmailCompletionWidget::setEmailBlackList(const QStringList &l
     mEmailList->setEmailBlackList(list);
 }
 
-void BlackListBalooEmailCompletionWidget::slotSaveChanges()
-{
-    // TODO avoid to save a lot.
-    const QHash<QString, bool> result = mEmailList->blackListItemChanged();
-    KSharedConfig::Ptr config = KSharedConfig::openConfig(QStringLiteral("kpimbalooblacklist"));
-    KConfigGroup group(config, "AddressLineEdit");
-    QStringList blackList = group.readEntry("BalooBackList", QStringList());
-    PimCommon::BlackListBalooEmailUtil util;
-    util.initialBlackList(blackList);
-    util.newBlackList(result);
-    blackList = util.createNewBlackList();
-    group.writeEntry("BalooBackList", blackList);
-    group.sync();
-    mEmailList->setEmailBlackList(blackList);
-    slotSearch();
-}
-
 void BlackListBalooEmailCompletionWidget::slotCheckIfUpdateBlackListIsNeeded()
 {
     const QHash<QString, bool> result = mEmailList->blackListItemChanged();
@@ -232,30 +302,6 @@ void BlackListBalooEmailCompletionWidget::slotCheckIfUpdateBlackListIsNeeded()
     }
 }
 
-void BlackListBalooEmailCompletionWidget::save()
-{
-    const QString domain = mExcludeDomainLineEdit->text().remove(QLatin1Char(' '));
-    const QStringList newExcludeDomain = domain.split(QLatin1Char(','), Qt::SkipEmptyParts);
-    bool needToSave = (mOriginalExcludeDomain != newExcludeDomain);
-    KSharedConfig::Ptr config = KSharedConfig::openConfig(QStringLiteral("kpimbalooblacklist"));
-    KConfigGroup group(config, "AddressLineEdit");
-    const QHash<QString, bool> result = mEmailList->blackListItemChanged();
-    if (!result.isEmpty()) {
-        needToSave = true;
-        QStringList blackList = group.readEntry("BalooBackList", QStringList());
-        PimCommon::BlackListBalooEmailUtil util;
-        util.initialBlackList(blackList);
-        util.newBlackList(result);
-        blackList = util.createNewBlackList();
-        group.writeEntry("BalooBackList", blackList);
-    }
-    if (needToSave) {
-        group.writeEntry("ExcludeDomain", newExcludeDomain);
-        group.sync();
-    }
-    config->reparseConfiguration();
-}
-
 void BlackListBalooEmailCompletionWidget::slotLinkClicked(const QString &link)
 {
     if (link == QLatin1String("more_result")) {
